
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: PythonLibrary(src/python/twitter/common/rpc/BUILD:rpc-packaged)

from setuptools import setup

setup(**
{   'classifiers': [   'Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: OS Independent',
                       'Programming Language :: Python'],
    'description': 'twitter.common thrift helpers including finagle and ssl transports.',
    'install_requires': [   'twitter.common.lang==0.1.0',
                            'thrift==0.9.0',
                            'thrift'],
    'license': 'Apache License, Version 2.0',
    'name': 'twitter.common.rpc',
    'namespace_packages': [   'gen',
                              'gen.twitter',
                              'gen.twitter.finagle',
                              'twitter',
                              'twitter.common'],
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': [   '.',
                    'gen',
                    'gen.twitter',
                    'gen.twitter.finagle',
                    'gen.twitter.finagle.thrift',
                    'twitter',
                    'twitter.common',
                    'twitter.common.rpc',
                    'twitter.common.rpc.finagle',
                    'twitter.common.rpc.transports'],
    'url': 'https://github.com/twitter/commons',
    'version': '0.1.0',
    'zip_safe': True}
)
